import cv2
import numpy as np
import os

import numpy as np
import pandas as pd
from PIL import Image

from vis.run import run
from vis.user_classes import *

# # define directory path
# data_dir = '/Users/amitosi/PycharmProjects/chester/chester/data/videos/nature.mp4'
# data_dir = '/Users/amitosi/PycharmProjects/chester/chester/data/videos/VID-20230316-WA0001.mp4'
# data_dir = '/Users/amitosi/PycharmProjects/chester/chester/data/videos/wedding.mp4'
data_dir = '/Users/amitosi/PycharmProjects/chester/chester/data/videos/protest.mp4'
# Create a video capture object
cap = cv2.VideoCapture(data_dir)

# Define image size
image_shape = (3, 4096*2, 4096*2)

vis_collector = run(video=cap,
                    frame_per_second=1,
                    image_shape=image_shape, plot_sample=16,
                    get_grayscale=False, get_reveresed_video=False,
                    detect_video_objects=True,
                    get_zoomed_video=False, zoom_factor=1.5, zoom_center=(500, 500),
                    get_frames_description=False, detect_frame_objects=True, detect_faces=False, plot=False)



import IPython
import torch
import torchaudio
from torchaudio.utils import download_asset

from udi.run import run  # Replace 'your_module' with the module name where the 'run' function is defined

# Print torch and torchaudio versions
print(torch.__version__)
print(torchaudio.__version__)

# Set random seed
torch.random.manual_seed(0)

# Set device to use CUDA if available, otherwise use CPU
device = torch.device("cuda" if torch.cuda.is_available() else "cpu")

# Download speech file
SPEECH_FILE = download_asset("tutorial-assets/Lab41-SRI-VOiCES-src-sp0307-ch127535-sg0042.wav")

# Display the audio file
IPython.display.Audio(SPEECH_FILE)

# Load the waveform and sample rate from the speech file
waveform, sample_rate = torchaudio.load(SPEECH_FILE)
waveform = waveform.to(device)

# Resample the waveform if sample_rate is not equal to the model's sample rate (16kHz)
if sample_rate != 16000:
    waveform = torchaudio.functional.resample(waveform, sample_rate, 16000)

# Create lists with the waveform and sample rate as their single elements
audio_data = [waveform.squeeze().cpu().numpy()]
sample_rates = [16000]

# Call the 'run' function with the audio data and sample rates
udi_collector = run(audio_data=audio_data, sample_rates=sample_rates, labels=None, speech_to_text=True)

# Print the predicted transcription from the udi_collector dictionary
print("Predicted Transcription:", udi_collector["speech to text"][0])



from diamond import run as diamond_run
from vis.grayscale.grayscale import VideoToGrayScale
from vis.object_video_tracker.object_tracker import ObjectVideoTracker
from vis.reversed_order.video_revereser import VideoReverser
from vis.user_classes import VideoData
from vis.video_zoom.zoom_class import VideoZoomer


def run(video,
        image_shape=None,
        frame_per_second=None,
        plot=True,
        get_frames_description=False,
        detect_frame_objects=False,
        detect_video_objects=False,
        detect_faces=False,
        get_grayscale=False,
        get_reveresed_video=False,
        get_zoomed_video=False, zoom_factor=1.5, zoom_center=None,
        plot_sample=16
        ):
    vis_collector = {}

    ## pre
    if detect_video_objects:
        detect_frame_objects = True

    video_data = VideoData(cap=video, image_shape=image_shape, frame_per_second=frame_per_second)
    vis_collector["video_data"] = video_data

    if get_grayscale:
        video_gray_scale = VideoToGrayScale(video_data)
        video_gray_scale.run(play=plot)
        vis_collector["gray scaled frames"] = video_gray_scale.grayscale_frames

    if get_reveresed_video:
        video_reverse = VideoReverser(video_data)
        video_reverse.run(play=plot)
        vis_collector["reversed video frames"] = video_reverse.reversed_frames

    if get_zoomed_video:
        video_zoomed = VideoZoomer(video_data, zoom_factor=zoom_factor, zoom_center=zoom_center)
        video_zoomed.run(play=plot)
        vis_collector["zoomed frames"] = video_zoomed.zoomed_frames

    diamond_collector = diamond_run.run(images=video_data.images,
                                        image_shape=image_shape,
                                        plot_sample=plot_sample,
                                        get_image_description=get_frames_description,
                                        get_object_detection=detect_frame_objects,
                                        detect_faces=detect_faces,
                                        is_augment_data=False, is_post_model_analysis=False, is_train_model=False,
                                        plot=plot)

    vis_collector.update(diamond_collector)

    if detect_video_objects:
        images = video_data.images
        object_bounding_box = vis_collector["object detection"]
        video_object_detector = ObjectVideoTracker(images=images, object_bounding_box=object_bounding_box,
                                                   fps=video_data.frame_per_second)
        video_object_detector.play()

    return vis_collector



import cv2


class ObjectVideoTracker:
    def __init__(self, images, object_bounding_box, fps=None):
        self.images = images
        self.object_bounding_box = object_bounding_box
        self.fps = fps

    def play(self):
        # Get the dimensions of the first frame
        height, width, channels = self.images[0].shape
        # Create a VideoWriter object to write the output video
        fourcc = cv2.VideoWriter_fourcc(*'mp4v')
        out = cv2.VideoWriter('video_object_tracker.avi', fourcc, self.fps or len(self.images), (width, height))
        # Process each frame in the input video
        for i, image in enumerate(self.images):
            bboxes = self.object_bounding_box[i]
            for bbox in bboxes:
                label = bbox['label']
                score = bbox['score']
                x1, y1, x2, y2 = bbox['box']['xmin'], bbox['box']['ymin'], bbox['box']['xmax'], bbox['box']['ymax']
                # Draw a rectangle around the detected object in the frame
                cv2.rectangle(image, (x1, y1), (x2, y2), (0, 255, 0), 2)
                # Put a label and score on the top-left corner of the bounding box
                text = f"{label} ({score:.2f})"
                cv2.putText(image, text, (x1, y1 - 10), cv2.FONT_HERSHEY_SIMPLEX, 0.5, (0, 255, 0), 1, cv2.LINE_AA)
            # Write the frame with the bounding boxes to the output video
            out.write(image)
            # Show the frame with the bounding boxes
            cv2.imshow('Object Tracker', image)
            if cv2.waitKey(int(1000 / (self.fps or len(self.images)))) & 0xFF == ord('q'):
                break
        # Release the VideoWriter object and close the output file
        out.release()
        cv2.destroyAllWindows()



import openai

from databots.code_to_file.code_to_file import write_code_to_file, run_python_file
from databots.utils import MODEL_ENGINE, api_key

openai.api_key = api_key


def modify_code(text, file_path, temperature=0.99, max_tokens=500):
    conversation = "This is my code:\n"
    code = open(file_path).read() + "\n"
    conversation = conversation + code + "\n"
    conversation = conversation + "Modify the code to do the following: \n" + text + "\n"
    conversation = conversation + "In your answer, re-write the whole file, and keep the same logic"

    response = openai.Completion.create(
        engine=MODEL_ENGINE,
        prompt=conversation,
        temperature=temperature,
        max_tokens=max_tokens
    )
    print(conversation)
    response = response.choices[0].text + "\n"
    write_code_to_file(response, file_path)
    return response, run_python_file(file_path)



import os
import pandas as pd
import librosa
import numpy as np

from chester.run.full_run import run
from chester.run.user_classes import Data, ModelRun
from udi.run import run

# data_dir = "/Users/amitosi/PycharmProjects/chester/udi/data"
data_dir = "/Users/amitosi/PycharmProjects/chester/udi/data_yes_no"
labels = []
audio_data = []
sample_rates = []

for subdir, dirs, files in os.walk(data_dir):
    for file in files:
        try:
            filepath = os.path.join(subdir, file)
            label = os.path.basename(subdir)
            labels.append(label)
            audio_dat, sample_rate = librosa.load(filepath)
            audio_data.append(audio_dat)
            sample_rates.append(sample_rate)
        except Exception as e:
            print(f"Error encountered while parsing {file}: {e}")
            labels.pop()

# udi_collector = run(audio_data=audio_data, sample_rates=sample_rates, labels=labels, n_mfcc=100)



import json
import re

import pandas as pd

pd.set_option('display.max_rows', None)
pd.set_option('display.max_columns', None)


class FlightDataParser:
    def __init__(self):
        self.df = pd.DataFrame()

    def from_json_to_pandas(self, json_file):
        with open(json_file, 'r') as f:
            flight_data = json.load(f)

        self.df = pd.DataFrame(flight_data)
        self.df['created_at'] = pd.to_datetime(self.df['created_at'])
        self.df.sort_values(by='created_at', ascending=False, inplace=True)
        # self.df.drop_duplicates(subset=['from_date', 'to_date', 'destination'], keep='first', inplace=True)
        self.df.reset_index(drop=True, inplace=True)
        self.df['from_date'] = pd.to_datetime(self.df['from_date'], format='%b %d', errors='coerce')
        self.df['from_date'] = self.df['from_date'].apply(lambda x: x.replace(year=2023) if pd.notnull(x) else x)
        self.df['to_date'] = pd.to_datetime(self.df['to_date'], format='%b %d', errors='coerce')
        self.df['to_date'] = self.df['to_date'].apply(lambda x: x.replace(year=2023) if pd.notnull(x) else x)

    def parse_flights(self, text):
        flight_pattern = re.compile(
            r'(\d{1,2}:\d{2}\s*[AP]M)(\s*\+\d)?\s*–\s*'
            r'(\d{1,2}:\d{2}\s*[AP]M)(\s*\+\d)?\n'
            r'(?:Separate tickets booked together)?\s*([^,\n]+)(?:,)?[^,\n]*\n'
            r'(\d\s*hr\s*\d{1,2}\s*min)'
        )

        flights = []
        for match in flight_pattern.finditer(text):
            departure_time = match.group(1)
            if match.group(2):
                departure_time += match.group(2)
            arrival_time = match.group(3)
            if match.group(4):
                arrival_time += match.group(4)
            airline = match.group(5).strip()
            duration = match.group(6)
            price_pattern = re.compile(r'₪(\d+,*\d*)')
            price_match = price_pattern.search(text, match.end())
            if price_match:
                price = int(price_match.group(1).replace(',', ''))
                flights.append({
                    'departure_time': departure_time,
                    'arrival_time': arrival_time,
                    'airline': airline,
                    'duration': duration,
                    'origin': 'TLV',
                    'destination': 'ATH',
                    'price': price
                })

        return pd.DataFrame(flights)

    def explode_flights(self):
        parsed_flights = []
        for index, row in self.df.iterrows():
            flights_df = self.parse_flights(row['content'])
            flights_list = flights_df.to_dict('records')

            for flight in flights_list:
                flight['from_date'] = row['from_date']
                flight['to_date'] = row['to_date']
                flight['destination'] = row['destination']
                flight['created_at'] = row['created_at']

            parsed_flights.extend(flights_list)

        columns = ['from_date', 'to_date', 'destination', 'created_at', 'departure_time', 'arrival_time', 'airline',
                   'duration', 'origin', 'price']
        self.df = pd.DataFrame(parsed_flights, columns=columns)


def load_flight_data():
    json_file_path = "/Users/amitosi/PycharmProjects/databots/databots/researcher/google_flights/flights.json"
    parser = FlightDataParser()
    parser.from_json_to_pandas(json_file_path)
    parser.explode_flights()
    return parser.df


df = load_flight_data()

# Performing EDA for Various Features
from chester.run import full_run as fr
from chester.run import user_classes as uc

fr.run(uc.Data(df=df))



import numpy as np
import os
import random
import numpy as np
import pandas as pd
# exec(open("/Users/amitosi/PycharmProjects/chester/diamond/manual_run.py").read())
from PIL import Image

from diamond.run import run
from diamond.user_classes import ImagesAugmentationInfo, ImageModel, ImageModels

# # define directory path
# data_dir = '/Users/amitosi/PycharmProjects/chester/chester/data/pics_ex'
# data_dir = '/Users/amitosi/PycharmProjects/chester/chester/data/pics_ex2'
data_dir = '/Users/amitosi/PycharmProjects/chester/chester/data/pic_ex3'

# # define image size
img_size = (1024, 1024)
# img_size = (640, 480)
# img_size = (32, 32)
#
# define empty arrays to store images and labels
images = []
labels = []
# Loop through each file in the pizza folder and add it to the images list with label 0 (pizza)
for filename in os.listdir(data_dir):
    image = Image.open(os.path.join(data_dir, filename))

    # # Define the area of the image to zoom in on
    # left = 50
    # right = 4000
    # top = left
    # bottom = right
    #
    # # Crop the image to the defined area
    # cropped_img = image.crop((left, top, right, bottom))
    #
    # # Resize the cropped image to make it appear zoomed in
    # zoomed_img = cropped_img.resize((600, 600))
    #
    # # Show the zoomed image
    # # zoomed_img.show()

    # images.append(zoomed_img)
    images.append(image)

# images = random.sample(images, 7)

new_images = []

# for image in images:
#     img = image.resize(img_size)
#     img_array = np.array(img)
#     new_images.append(img_array)

# new_images = np.array(new_images)
print("Total pics", len(new_images))

# labels = pd.read_csv(data_dir + "/labels.csv").fillna('UNK')['Category']

# # convert the lists to NumPy arrays
# images = np.array(images[0:100])
# print(np.unique(labels))
# labels = np.array(labels[0:100])

image_shape = (3, 1024, 1024)
# image_shape = (3, 480, 640)


image_model_list = [
    ImageModel(network_name="EfficientNetB0",
               batch_size=64 * 64 * 16,
               num_epochs=1,
               optimizer_params={'lr': 0.005},
               dropout=0.7)]
image_models = ImageModels(image_model_list=image_model_list)

diamond_collector = run(images=images,
                        image_shape=image_shape,
                        # labels=labels,
                        # get_image_description=True,
                        get_object_detection=True,
                        # detect_faces=True,
                        is_augment_data=False,
                        image_augmentation_info=ImagesAugmentationInfo(aug_prop=0.7),
                        is_train_model=False, image_models=image_models,
                        is_post_model_analysis=False,
                        plot=True)



import os
import librosa
import torch
import numpy as np

# Load the SERNet model
model = torch.hub.load('pyannote/pyannote-audio', 'SER_0')

# Define a list of emotion labels
emotions = ['neutral', 'calm', 'happy', 'sad', 'angry', 'fearful', 'disgust', 'surprised']

# Define the path to the directory containing the audio files
data_dir = "/Users/amitosi/PycharmProjects/chester/udi/emotion"

# Define empty lists to store the emotion labels and probabilities
emotion_labels = []
emotion_probs = []

# Set the model to evaluation mode
model.eval()

# Disable gradient calculation to speed up inference
torch.set_grad_enabled(False)

# Iterate through the audio files in the directory
for file in os.listdir(data_dir):
    try:
        # Load the audio data and resample it to 16 kHz (the same sample rate used during training)
        filepath = os.path.join(data_dir, file)
        audio_dat, sample_rate = librosa.load(filepath, sr=16000)
        audio_dat = librosa.resample(audio_dat, sample_rate, 16000)

        # Extract the features from the audio data using the SERNet model
        features = model({'audio': audio_dat})

        # Compute the probability distribution over the emotion classes
        probs = torch.nn.functional.softmax(features['log_likelihood']).numpy()[0]

        # Choose the emotion with the highest probability as the predicted emotion
        pred_emotion = emotions[np.argmax(probs)]

        # Append the predicted emotion label and probability to the respective lists
        emotion_labels.append(pred_emotion)
        emotion_probs.append(probs)
    except Exception as e:
        print(f"Error encountered while parsing {file}: {e}")



import datetime

MODEL_ENGINE = "text-davinci-003"  # or "text-curie-001" for Curie
api_key = "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"


def log(text):  # define a function, `log`
    current_time = datetime.datetime.now()  # set the current time variable to the current time
    print(f'{current_time}: {text}')  # prints the currentTime and the given `text` parameter



import cv2

data_dir = '/Users/amitosi/PycharmProjects/chester/video_object_tracker.avi'

# Path to the input video file

# Create a VideoCapture object to read the input video
cap = cv2.VideoCapture(data_dir)

# Loop through the video frames
while cap.isOpened():
    # Read a frame from the video
    ret, frame = cap.read()

    # If the frame was read successfully
    if ret:
        # Display the frame
        cv2.imshow('Video', frame)

        # Exit if the 'q' key is pressed
        if cv2.waitKey(500) & 0xFF == ord('q'):
            break
    else:
        # Exit if there are no more frames
        break

# Release the VideoCapture object and close the window
cap.release()
cv2.destroyAllWindows()



import math
import random

import matplotlib.pyplot as plt
import numpy as np
import pandas as pd
import torch
from PIL import Image
from torch.utils.data import TensorDataset, DataLoader
from tqdm import tqdm


class ImagesData:
    def __init__(self, images, labels, validation_prop, image_shape, label_dict, for_model_training=True):
        self.images = images
        self.raw_images = self.images
        self.raw_image = None
        self.image_shape = image_shape
        # print("image shape origin", self.image_shape)
        self.is_colored = True if len(self.image_shape) > 2 else False

        self.format_images()  # format images
        self.labels = labels
        self.validation_prop = validation_prop
        self.label_dict = label_dict
        self.for_model_training = for_model_training
        self.validate()  # validation
        self.problem_type = self.get_problem_type()
        self.label_hanlder()
        self.images_to_show = None
        self.image_to_show()
        if self.for_model_training:
            self.images_train, self.labels_train, self.images_val, self.labels_val = self.split()
        if self.is_colored:
            print("Colored image")
        else:
            print("Grayscale image")

    def format_images(self):
        # print("Formatting images of type", type(self.images))
        np_images = []
        image_list = []

        if isinstance(self.images, pd.DataFrame):
            # Convert DataFrame to numpy array
            np_images = self.images.to_numpy()

            # Convert each numpy array to PIL Image object and add to list
            for img_arr in np_images:
                img = Image.fromarray(img_arr)
                image_list.append(img)

        elif isinstance(self.images, list):
            # Loop through each image and convert to numpy array and PIL Image object
            for idx, image in enumerate(self.images):
                if isinstance(image, str):
                    # if image is a file path, open it using PIL and convert to numpy array
                    with Image.open(image) as img:
                        img_array = np.array(img)
                elif isinstance(image, np.ndarray):
                    # if image is a numpy array, use it directly
                    img_array = image
                else:
                    try:
                        img_array = np.array(image)
                    except:
                        img_array = image

                # Append PIL Image object and numpy array to lists
                image_list.append(Image.fromarray(img_array))
                np_images.append(img_array)

        elif isinstance(self.images, np.ndarray):
            # Convert numpy array to PIL Image object
            for img_arr in self.images:
                if self.is_colored:
                    mode = 'RGB'
                    height, width = sorted(self.image_shape, reverse=True)[:2]
                    img_arr = img_arr.reshape((height, width, 3))
                else:
                    mode = 'L'  # L stands for grayscale mode
                    img_arr = img_arr.reshape(self.image_shape)
                img = Image.fromarray(img_arr, mode=mode)
                image_list.append(img)
                np_images.append(img_arr)
        else:
            raise TypeError("Unsupported image format:", type(self.images))

        # Update the instance variables with the formatted image data
        self.images = np.array(np_images)
        self.raw_image = image_list[0]
        self.raw_images = image_list

    def validate(self):
        if self.labels is None:
            pass
        assert len(self.images) == len(self.labels), "# of Images must be == # of labels"

    def image_to_show(self):
        try:
            images = self.images.reshape((-1,) + self.image_shape)
            if self.is_colored:
                self.images_to_show = np.transpose(images, (0, 2, 3, 1))
            else:
                self.images_to_show = np.transpose(images, (0, 1, 2))
        except:
            pass

    def label_hanlder(self):
        if isinstance(self.labels, pd.DataFrame):
            self.labels = self.labels.to_numpy()

    def split(self):
        assert 0 <= self.validation_prop < 0.5, "validation proportion should be in range (0, 0.5)"
        print("Before splitting, these are the shapes")
        print("images:", self.images.shape)

        num_images = len(self.images)
        num_val_images = int(num_images * self.validation_prop)
        num_train_images = num_images - num_val_images

        # Shuffle the images and labels before splitting
        shuffled_indices = np.random.permutation(num_images)
        shuffled_images = np.array(self.images)[shuffled_indices]
        shuffled_labels = self.labels[shuffled_indices]

        # Split the images and labels into training and validation sets
        train_images = shuffled_images[:num_train_images]
        train_labels = shuffled_labels[:num_train_images]
        val_images = shuffled_images[num_train_images:]
        val_labels = shuffled_labels[num_train_images:]

        # Convert to correct shape
        train_images = train_images.reshape((-1,) + self.image_shape)
        val_images = val_images.reshape((-1,) + self.image_shape)

        print("After splitting, these are the shapes")
        print("train_images:", train_images.shape)
        print("val_images:", val_images.shape)

        return train_images, train_labels, val_images, val_labels

    def create_data_loaders(self, batch_size):
        labels_train = np.array(self.labels_train)
        train_dataset = TensorDataset(torch.from_numpy(self.images_train), torch.from_numpy(labels_train))
        train_loader = DataLoader(train_dataset, batch_size=batch_size, shuffle=True)

        labels_val = np.array(self.labels_val)
        val_dataset = TensorDataset(torch.from_numpy(self.images_val), torch.from_numpy(labels_val))
        val_loader = DataLoader(val_dataset, batch_size=batch_size, shuffle=False)

        # add tqdm progress bar to train_loader
        train_loader = tqdm(train_loader, total=len(train_loader))

        return train_loader, val_loader

    def get_splits(self):
        return self.images_train, self.labels_train, self.images_val, self.labels_val

    @staticmethod
    def get_problem_type():
        return "classification"

    # def plot_images(self, plot_sample):
    #     print("Total Images:", len(self.images))
    #     if self.images_to_show is None:
    #         self.images_to_show = self.images
    #     images = self.images_to_show
    #     num_images = len(images)
    #     plot_sample = min(plot_sample, num_images)
    #     if num_images > 100:
    #         image_indices = range(num_images - plot_sample + 1, num_images)
    #     else:
    #         image_indices = random.sample(range(num_images), plot_sample)
    #
    #     num_cols = int(math.floor(math.sqrt(len(image_indices))))
    #     num_rows = int(math.ceil(len(image_indices) / num_cols))
    #
    #     fig, ax = plt.subplots(num_rows, num_cols, figsize=(12, 12))
    #     for i, index in enumerate(image_indices):
    #         row = i // num_cols
    #         col = i % num_cols
    #         ax[row, col].imshow(images[index])
    #         ax[row, col].axis('off')
    #         if self.labels is not None:
    #             ax[row, col].set_title(str(self.labels[index]))
    #     plt.show()

    def plot_images(self, plot_sample):
        print("Total Images:", len(self.images))
        if self.images_to_show is None:
            self.images_to_show = self.images
        images = self.images_to_show
        num_images = len(images)
        plot_sample = min(plot_sample, num_images)
        if num_images > 100:
            image_indices = range(num_images - plot_sample + 1, num_images)
        else:
            image_indices = random.sample(range(num_images), plot_sample)

        if len(image_indices) == 1:
            fig, ax = plt.subplots(figsize=(12, 12))
            ax.imshow(images[image_indices[0]])
            ax.axis('off')
            if self.labels is not None:
                ax.set_title(str(self.labels[image_indices[0]]))
        else:
            num_cols = int(math.floor(math.sqrt(len(image_indices))))
            num_rows = int(math.ceil(len(image_indices) / num_cols))
            fig, ax = plt.subplots(num_rows, num_cols, figsize=(12, 12))
            for i, index in enumerate(image_indices):
                row = i // num_cols
                col = i % num_cols
                ax[row, col].imshow(images[index])
                ax[row, col].axis('off')
                if self.labels is not None:
                    ax[row, col].set_title(str(self.labels[index]))
        plt.show()


class ImagesAugmentationInfo:

    def __init__(self, aug_types=None, aug_prop=0.05):
        self.aug_types = aug_types
        self.aug_prop = aug_prop
        # calc
        if self.aug_types is None:
            self.aug_types = ["zoom", "rotate"]


class ImageModel:
    def __init__(self,
                 network_name="EfficientNetB0",
                 remove_num_last_layers=1,
                 batch_size=64,
                 num_epochs=3,
                 optimizer_params=None,
                 dropout=0.5
                 ):
        self.network_name = network_name
        self.remove_last_layers_num = remove_num_last_layers
        self.batch_size = batch_size
        self.num_epochs = num_epochs
        self.optimizer_params = optimizer_params
        self.dropout = dropout
        if self.optimizer_params is None:
            self.optimizer_params = {"lr": 0.001, 'weight_decay': 0.0001}
        self.network_parameters = self.optimizer_params.copy()
        self.network_parameters["_model_name"] = self.network_name
        self.network_parameters["_num_epochs"] = self.num_epochs
        self.network_parameters["_batch_size"] = self.batch_size
        self.network_parameters["_dropout"] = self.dropout
        self.network_parameters["_remove_last_layers_num layers"] = self.remove_last_layers_num


class ImageModels:
    def __init__(self, image_model_list=None):
        self.image_model_list = image_model_list
        if image_model_list is None:
            self.image_model_list = [ImageModel()]
        self.n_models = len(self.image_model_list)


class ImagePostModelSpec:
    def __init__(self, plot,
                 is_compare_models=True,
                 is_confusion_matrix=True,
                 is_precision_recall=True):
        self.plot = plot
        self.is_compare_models = is_compare_models
        self.is_confusion_matrix = is_confusion_matrix
        self.is_precision_recall = is_precision_recall


class ImageDescriptionSpec:
    def __init__(self, max_length=16, num_beams=4):
        self.max_length = max_length
        self.num_beams = num_beams
        self.gen_kwargs = {"max_length": self.max_length, "num_beams": self.num_beams}



import torch
from PIL import Image
from PIL import ImageDraw
from matplotlib import pyplot as plt
from transformers import pipeline

from diamond.user_classes import ImagesData


class ImageObjectDetection:
    def __init__(self,
                 images_data: ImagesData,
                 plot=True, plot_sample=10,
                 diamond_collector=None):
        self.images_data = images_data
        self.plot = plot
        self.plot_sample = plot_sample
        self.device = torch.device('cuda' if torch.cuda.is_available() else 'cpu')
        self.label_dict = self.images_data.label_dict
        self.model = None
        # TODO: later add support for labels (labels VS description?)
        self.diamond_collector = {} if diamond_collector is None else diamond_collector
        self.images = self.images_data.raw_images
        self.load_model()

    def load_model(self):
        self.model = get_image_object_detection_model()

    def detect_objects(self):
        bounding_boxes = []
        for image in self.images:
            # Run object detection on image
            detections = self.model(image)

            # Append detections to list
            bounding_boxes.append(detections)

        return bounding_boxes

    def plot_detected_objets(self):
        if self.plot:
            try:
                for i in range(len(self.images_data.images_to_show)):
                    try:
                        image = Image.fromarray(self.images_data.images_to_show[i])
                    except:
                        image = self.images_data.images_to_show[i]
                    bounding_boxes = self.diamond_collector["object detection"][i]
                    lables = []
                    # Draw bounding boxes and labels on original image
                    draw = ImageDraw.Draw(image)
                    for prediction in bounding_boxes:
                        box = prediction["box"]
                        label = prediction["label"]
                        lables.append(label)
                        draw.rectangle([box["xmin"], box["ymin"], box["xmax"], box["ymax"]], outline="red", width=12)
                        draw.text((box["xmin"], box["ymin"] - 20), label, fill="black")
                    # Display image with bounding boxes
                    fig, ax = plt.subplots(figsize=(15, 11))
                    plt.title(lables)
                    plt.imshow(image)
                    plt.show()
                    plt.close()
                    if i > self.plot_sample:
                        return None
            except Exception as e:
                print(f"Error while running model: {e}")

    def run(self):
        self.diamond_collector["object detection"] = self.detect_objects()
        if self.plot:
            self.plot_detected_objets()


def get_image_object_detection_model():
    model_name = "facebook/detr-resnet-50"
    model_revision = "main"
    model = pipeline("object-detection", model=model_name, revision=model_revision)
    return model



import os

source_root = os.getcwd()  # Get the current working directory as the source root
folders = []

# Recursively search for all sub-folders under the source root
for root, dirnames, filenames in os.walk(source_root):
    for dirname in dirnames:
        folders.append(os.path.join(root, dirname))

print(folders)



import math

import cv2
from matplotlib import pyplot as plt


class VideoData:
    def __init__(self, cap, frame_per_second=None, image_shape=None):
        self.video = cap
        self.image_shape = image_shape
        self.frame_per_second = frame_per_second
        self.images = None
        self.extract_images()

    def extract_images(self):
        # Get the total number of frames in the video
        total_frames = int(self.video.get(cv2.CAP_PROP_FRAME_COUNT))

        # If self.frame_per_second is None, extract all frames
        if self.frame_per_second is None:
            self.images = self._extract_all_frames(total_frames)

        # If self.frame_per_second is a value, extract that many frames per second
        else:
            self.images = self._extract_frames_per_second(total_frames)
        print("Extracted total", len(self.images), "frames")

    def _extract_all_frames(self, total_frames):
        images = []
        for i in range(total_frames):
            # Read the next frame
            ret, frame = self.video.read()

            # If the frame is not valid, break out of the loop
            if not ret:
                break

            # Add the frame to the images list
            images.append(frame)

        return images

    def _extract_frames_per_second(self, total_frames):
        images = []
        fps = self.video.get(cv2.CAP_PROP_FPS)
        duration_sec = total_frames / fps
        num_frames = int(self.frame_per_second * duration_sec)
        frames_to_skip = int(fps / self.frame_per_second)

        # Set the frame position to the beginning of the video
        self.video.set(cv2.CAP_PROP_POS_FRAMES, 0)

        # Loop through the frames and extract every nth frame
        for i in range(num_frames):
            # Skip frames as needed
            for j in range(frames_to_skip):
                ret = self.video.grab()
                if not ret:
                    break

            # Read the next frame
            ret, frame = self.video.retrieve()

            # If the frame is not valid, break out of the loop
            if not ret:
                break

            # Add the frame to the images list
            images.append(frame)

        return images

    def validate(self):
        pass

    def plot_video_images(self, plot_sample):
        total_images = len(self.images)
        plot_sample = min(plot_sample, total_images)
        num_rows = math.ceil(math.sqrt(plot_sample))
        num_cols = math.ceil(plot_sample / num_rows)

        sample_indices = set(range(total_images))  # Get a set of all image indices
        sample_indices = sorted(sample_indices,
                                key=lambda i: i % num_cols)  # Sort the indices by the image order in the video
        sample_indices = sample_indices[:plot_sample]  # Keep only the first `plot_sample` indices

        fig, axs = plt.subplots(num_rows, num_cols, figsize=(10, 10))

        for i, ax in enumerate(axs.flatten()):
            if i < plot_sample:
                index = sample_indices[i]
                image = self.images[index]
                ax.imshow(image)
                ax.set_xticks([])
                ax.set_yticks([])
                # ax.set_title("{:.1f}%".format((index + 1) / plot_sample * 100))

        plt.tight_layout()
        plt.show()



import cv2


class VideoZoomer:
    def __init__(self, video_data, zoom_factor, zoom_center=None):
        self.video_data = video_data
        self.frames = self.video_data.images
        self.fps = self.video_data.frame_per_second
        self.zoom_factor = zoom_factor
        self.zoom_center = zoom_center
        self.zoomed_frames = None

    def create_zoomed_frames(self):
        """
        Zooms the frames by the specified zoom factor, centered on the specified zoom center (if provided).
        """
        zoomed_frame_list = []
        for frame in self.frames:
            height, width, _ = frame.shape
            new_width = int(width / self.zoom_factor)
            new_height = int(height / self.zoom_factor)

            # Calculate the x and y offsets to center the zoomed frame
            if self.zoom_center is None:
                x_offset = (width - new_width) // 2
                y_offset = (height - new_height) // 2
            else:
                x_offset = max(0, self.zoom_center[0] - (new_width // 2))
                y_offset = max(0, self.zoom_center[1] - (new_height // 2))

            cropped_frame = frame[y_offset:y_offset + new_height, x_offset:x_offset + new_width]
            resized_frame = cv2.resize(cropped_frame, (width, height), interpolation=cv2.INTER_LINEAR)
            zoomed_frame_list.append(resized_frame)

        self.zoomed_frames = zoomed_frame_list
        return zoomed_frame_list

    def run(self, play=False):
        if not self.zoomed_frames:
            self.create_zoomed_frames()

        height, width, _ = self.zoomed_frames[0].shape
        size = (width, height)

        video = cv2.VideoWriter('zoomed_output_video.avi', cv2.VideoWriter_fourcc(*'XVID'), self.fps, size)

        for img in self.zoomed_frames:
            video.write(img)

        video.release()
        if play:
            try:
                self.play_zoomed_video()
            except:
                print("Cannot play video on collab, you can read the video and use the vis module to plot images")
        return video

    def play_zoomed_video(self):
        cap = cv2.VideoCapture('zoomed_output_video.avi')

        while cap.isOpened():
            ret, frame = cap.read()

            if not ret:
                break

            cv2.imshow('Zoomed Video', frame)

            if cv2.waitKey(int(1000 / self.fps)) & 0xFF == ord('q'):
                break

        cap.release()
        cv2.destroyAllWindows()



import ast
import io
import os
import sys
from tempfile import NamedTemporaryFile

import black
import openai
import pycodestyle
import pyflakes.api

from databots.code_development.develop import *
from databots.code_development.error_handling import qa_fix_comments
from databots.code_health.clean_not_code import clean_python_code
from databots.utils import *

openai.api_key = api_key


class FormatFile:
    def __init__(self, file_path: str, is_black=True, is_pyflake=True, is_clean_no_code=True):
        self.file_path = file_path
        self.is_black = is_black
        self.is_pyflake = is_pyflake
        self.is_clean_no_code = is_clean_no_code
        self.code_comments = self.get_code_comments()

    def get_code_comments(self):
        report1 = check_file_for_debugger_statements(filepath=self.file_path)
        report2 = get_pyflakes_report(file_path=self.file_path)
        return report1 + "\n" + report2

    def run(self):
        # clean_python_code(self.file_path)
        if self.is_black:
            format_file_with_black(self.file_path)
        # if self.is_pyflake:
        #     qa_fix_comments(comments=self.code_comments, file_path=self.file_path)
        # if self.is_black:
        #     format_file_with_black(self.file_path)


def format_file_with_black(file_path: str) -> None:
    """Formats the file at `file_path` using the Black code formatter."""
    with open(file_path, "r") as f:
        file_contents = f.read()

    # Format the file using Black
    formatted_contents = black.format_file_contents(
        file_contents,
        fast=False,
        mode=black.Mode(),
    )

    # Write the formatted contents back to the file
    with open(file_path, "w") as f:
        f.write(formatted_contents)


def get_pyflakes_report(file_path: str) -> str:
    """Checks the Python file at `file_path` using PyFlakes and returns the report."""
    with open(file_path, 'r') as f:
        tree = compile(f.read(), file_path, 'exec', ast.PyCF_ONLY_AST)

    report = io.StringIO()
    reporter = pyflakes.reporter.Reporter(report, file_path)

    pyflakes.api.check(tree, file_path, reporter)

    return report.getvalue()


def check_file_for_debugger_statements(filepath, pycodestyle_report=
"/Users/amitosi/PycharmProjects/chester/databots/code_health/pycodestyle_report.txt"):
    """Process file using pycodestyle Checker and return all errors."""

    if os.path.isfile(pycodestyle_report):
        with open(pycodestyle_report, "w") as f:
            pass
    else:
        print("File does not exist.")

    if not os.path.isfile(filepath):
        raise ValueError("Invalid file path")

    with open(filepath, "r") as f:
        code = f.read()

    test_file = NamedTemporaryFile(delete=False)
    test_file.write(code.encode())
    test_file.flush()

    # Redirect standard output to a file
    original_stdout = sys.stdout
    sys.stdout = open(pycodestyle_report, "a")

    errors = []
    lines = [line + "\n" for line in code.split("\n")]
    checker = pycodestyle.Checker(filename=test_file.name, lines=lines)

    num_errors = checker.check_all()
    if num_errors == 0:
        os.unlink(test_file.name)
        sys.stdout.close()
        sys.stdout = original_stdout
        return errors

    # Save the printed statements to the input file
    with open(filepath, "w") as f:
        f.write("".join(lines))

    os.unlink(test_file.name)
    sys.stdout.close()
    sys.stdout = original_stdout
    with open(pycodestyle_report, "r") as f:
        file_contents = f.read()
    return file_contents



import subprocess


def run_python_file(path: str):
    # will return None if not error, otherwise the error
    try:
        subprocess.run(['python', path], stdout=subprocess.PIPE, stderr=subprocess.PIPE, check=True)
        return None
    except subprocess.CalledProcessError as e:
        return e.stderr.decode()


def execute_code(path: str):
    # run the code
    result = subprocess.run(['python', path], stdout=subprocess.PIPE, stderr=subprocess.PIPE, check=True)
    print(result.stdout.decode())


def write_code_to_file(code: str, path: str):
    with open(path, 'w') as file:
        file.write(code)



from databots.code_health.clean_not_code import clean_python_code
from databots.code_health.code_organization import FormatFile
from databots.from_scratch import create_from_scratch
from databots.modify_file import modify

# initiate
file = "games.py"
file_path = "/Users/amitosi/PycharmProjects/chester/databots/games.py"

text = """
Write a function to clean text. include 3 tests
"""
# create_from_scratch(text, python_file=file)
#
# # modify
# text = """
# Add comments and docstring to the code
# """
# # modify(text=text, python_file=file)
#
# # review
# file = "/Users/amitosi/PycharmProjects/chester/chester/run/full_run.py"
# review_code(file_path=file)
#
text = """
fix bugs
"""
# FormatFile(file_path=file_path).run()
# modify(text=text, python_file=file)
#

# clean_python_code(file_path)



import re
import warnings

import torch
from transformers import Wav2Vec2ForCTC, Wav2Vec2Processor


# Use the model and processor as before

def warning_filter(message):
    typed_storage_warning = "TypedStorage is deprecated"
    weights_warning = "Some weights of .* were not initialized from the model checkpoint"

    if re.search(typed_storage_warning, str(message)) or re.search(weights_warning, str(message)):
        return False
    return True


warnings.filterwarnings("ignore")


class SpeechToText:

    def __init__(self, audio_data, sample_rates, model_name="facebook/wav2vec2-base-960h"):
        self.audio_data = audio_data
        self.sample_rates = sample_rates
        self.model_name = model_name
        self.device = torch.device("cuda" if torch.cuda.is_available() else "cpu")

    def run(self):

        if self.model_name == "facebook/wav2vec2-base-960h":
            processor = Wav2Vec2Processor.from_pretrained(self.model_name)
            model = Wav2Vec2ForCTC.from_pretrained(self.model_name).to(self.device)
        transcriptions = []

        for waveform, sample_rate in zip(self.audio_data, self.sample_rates):
            waveform_tensor = torch.tensor(waveform).to(self.device)
            input_values = processor(waveform_tensor, return_tensors="pt", padding=True,
                                     sampling_rate=sample_rate).input_values.to(self.device)

            with torch.inference_mode():
                logits = model(input_values).logits

            predicted_ids = torch.argmax(logits, dim=-1)
            transcription = processor.batch_decode(predicted_ids)
            transcriptions.append(transcription[0])

        return transcriptions



import os

import openai

from databots.code_development.error_handling import qa
from databots.code_health.code_organization import FormatFile
from databots.code_to_file.code_to_file import *
from databots.modify_code.code_modification import modify_code
from databots.utils import *

openai.api_key = api_key


def modify(text,
           file_prefix="/Users/amitosi/PycharmProjects/chester/databots",
           python_file="generated_code.py"):
    conversation_collector = []
    log(f"Original task: {text}")
    file_path = os.path.join(file_prefix, python_file)

    # modify code
    conversation, error = modify_code(text, file_path)
    conversation_collector.append(conversation)

    # Fix errors
    count_fixes = 1
    while error is not None:
        log("Found bugs in the code! Fixing...")
        conversation, error = qa(original_task=text, file_path=file_path, error=error,
                                 max_tokens=700)
        conversation_collector.append(conversation)
        count_fixes += 1
        if count_fixes > 7:
            print("Something is wrong, try re-submit the task please")
            break

    log("Code Ran With No Errors!")

    # Run code
    log("Code outputs:")
    execute_code(file_path)

    # Organize code
    log("Organizing Code Style")
    FormatFile(file_path=file_path).run()
    return conversation_collector





# install and import necessary libraries 
import numpy as np 
import pandas as pd 

# download and load the dataset 
data_path = "data_file.csv" # modify the path to the dataset 
try: 
    dataset = pd.read_csv(data_path) 
except FileNotFoundError: 
        print("FileNotFoundError: [Errno 2] No such file or directory: 'data_file.csv'") 
        print("Check if the file is located in the correct path and try again!")

# data cleaning and pre-processing 
dataset.dropna(inplace=True) 
dataset.drop_duplicates(inplace=True) 

# exploratory data analysis 
import matplotlib.pyplot as plt 
import seaborn as sns 
# plotting distribution, correlation graphs etc.

# apply data mining techniques
# e.g. clustering, classification 

# build machine learning models 
# e.g. regression, decision trees, random forests, neural networks

# evaluate the models 
# e.g. precision, recall, accuracy etc.



import torch
from PIL import Image
from PIL import ImageDraw
from matplotlib import pyplot as plt

from diamond.user_classes import ImagesData


class ImageFaceDetection:
    def __init__(self,
                 images_data: ImagesData,
                 plot=True, plot_sample=10,
                 diamond_collector=None):
        self.images_data = images_data
        self.plot = plot
        self.plot_sample = plot_sample
        self.device = torch.device('cuda' if torch.cuda.is_available() else 'cpu')
        self.label_dict = self.images_data.label_dict
        self.model = None
        self.diamond_collector = {} if diamond_collector is None else diamond_collector
        self.images = self.images_data.raw_images
        self.load_model()

    def load_model(self):
        self.model = get_face_detection_model()

    # TODO: GPU support?
    def detect_yolo_faces(self):
        bounding_boxes = []

        for image in self.images:
            if image.mode != "RGB":
                image = image.convert(mode="RGB")

            # Apply YOLO
            results = self.model(image)

            # Extract the bounding boxes of the detected faces
            faces = results.xyxy[0][results.xyxy[0][:, -1] == 0][:, :4].tolist()
            bounding_boxes.append(faces)
        return bounding_boxes

    def plot_detected_objects(self):
        if not self.plot:
            return None

        fig, axes = plt.subplots(nrows=len(self.images_data.images_to_show),
                                 figsize=(15, 11 * len(self.images_data.images_to_show)))

        for i, (image_data, ax) in enumerate(zip(self.images_data.images_to_show, axes)):
            try:
                image = Image.fromarray(image_data)
            except TypeError:
                image = image_data
            bounding_boxes = self.diamond_collector["face detection"][i]
            # we want to see all of them
            if len(bounding_boxes) == 0:
                continue

            labels = []
            draw = ImageDraw.Draw(image)
            for prediction in bounding_boxes:
                box = prediction
                label = "Object"
                labels.append(label)
                draw.rectangle([box[0], box[1], box[0] + box[2], box[1] + box[3]], outline="red", width=2)
                draw.text((box[0], box[1] - 15), label, fill="black")

            ax.imshow(image)
            ax.set_title(f"Objects detected: {len(bounding_boxes)}")
            ax.axis('off')

            if i >= self.plot_sample:
                break
        plt.show()
        plt.close()

    def run(self):
        self.diamond_collector["face detection"] = self.detect_yolo_faces()
        if self.plot:
            self.plot_detected_objects()


def get_face_detection_model():
    # Load the YOLOv5s model for face detection
    model = torch.hub.load('ultralytics/yolov5', 'yolov5s', pretrained=True)

    return model



import cv2


class VideoReverser:
    def __init__(self, video_data):
        self.video_data = video_data
        self.frames = self.video_data.images
        self.reversed_frames = None
        self.fps = self.video_data.frame_per_second

    def create_reversed_frames(self):
        """
        Reverses the order of the frames in the list.
        """
        reversed_frame_list = self.frames[::-1]
        self.reversed_frames = reversed_frame_list
        return reversed_frame_list

    def run(self, play=False):
        if not self.reversed_frames:
            self.create_reversed_frames()

        height, width, _ = self.reversed_frames[0].shape
        size = (width, height)

        video = cv2.VideoWriter('reversed_output_video.avi', cv2.VideoWriter_fourcc(*'XVID'), self.fps, size)

        for img in self.reversed_frames:
            video.write(img)

        video.release()
        if play:
            try:
                self.play_reversed_video()
            except:
                print("Cannot play video on collab, you can read the video and use the vis module to plot images")
        return video

    def play_reversed_video(self):
        cap = cv2.VideoCapture('reversed_output_video.avi')

        while cap.isOpened():
            ret, frame = cap.read()

            if not ret:
                break

            cv2.imshow('Reversed Video', frame)

            if cv2.waitKey(int(1000 / self.fps)) & 0xFF == ord('q'):
                break

        cap.release()
        cv2.destroyAllWindows()



import cv2


class ImageListToVideo:
    def __init__(self, image_list, fps=30, plot=False):
        self.image_list = image_list
        self.fps = fps
        self.plot = plot
        self.video = self.create_video()
        if self.plot:
            self.play_video()

    def create_video(self):
        if not self.image_list:
            raise ValueError("The image list is empty.")

        height, width, _ = self.image_list[0].shape
        size = (width, height)

        video = cv2.VideoWriter('output_video.avi', cv2.VideoWriter_fourcc(*'XVID'), self.fps, size)

        for img in self.image_list:
            video.write(img)

        video.release()
        return video

    def play_video(self):
        cap = cv2.VideoCapture('output_video.avi')

        while cap.isOpened():
            ret, frame = cap.read()

            if not ret:
                break

            cv2.imshow('Video', frame)

            if cv2.waitKey(int(1000 / self.fps)) & 0xFF == ord('q'):
                break

        cap.release()
        cv2.destroyAllWindows()



import openai

from databots.utils import *

openai.api_key = api_key


def review_code(file_path, text="", temperature=0.99, max_tokens=200):
    conversation = text + "\nThis is the code:\n"
    code = open(file_path).read() + "\n"
    conversation = conversation + code + "\n"

    conversation = conversation + "Review the code, code quality, code style." \
                                  "Suggest ideas for improvements. " \
                                  "Give practical advices"

    response = openai.Completion.create(
        engine=MODEL_ENGINE,
        prompt=conversation,
        temperature=temperature,
        max_tokens=max_tokens
    )

    review_response = response.choices[0].text + "\n"
    print(conversation)
    print("******* Answer:\n")
    print(review_response)

    return review_response



def clean_python_code(file_path):
    with open(file_path, 'r') as file:
        content = file.readlines()

    cleaned_code = []

    for line in content:
        stripped_line = line.strip()
        if stripped_line.startswith('#') or is_python_code(stripped_line):
            cleaned_code.append(line)

    with open(file_path, 'w') as cleaned_file:
        cleaned_file.writelines(cleaned_code)

        # close the file after writing
        cleaned_file.close()


def is_python_code(line):
    try:
        compile(line, '<string>', 'exec')
        return True
    except (SyntaxError, TypeError, ValueError):
        return False



# import necessary libraries
import numpy as np
import pandas as pd
from catboost import CatBoostClassifier
from sklearn.datasets import fetch_openml
from sklearn.model_selection import train_test_split

# load mnist dataset
X, y = fetch_openml("mnist_784", version=1, return_X_y=True)

# split the dataset into training and validation sets
X_train, X_val, y_train, y_val = train_test_split(X, y, test_size=0.2)

# instantiate and fit the model
model = CatBoostClassifier(
    task_type="CPU", loss_function="MultiClass", n_estimators=500
)

model.fit(X_train, y_train)

# report accuracy on the validation set
print("Accuracy score on validation set: {:0.2f}".format(model.score(X_val, y_val)))



import librosa
import numpy as np
import pandas as pd

from chester.run.full_run import run as chester_run
from chester.run.user_classes import ModelRun, Data
from udi.speech_to_text.speech_to_text_class import SpeechToText


def run(audio_data, sample_rates, n_mfcc=40, labels=None,
        is_pre_model=True,
        is_train_model=True, model_run: ModelRun = None,
        is_post_model=True,
        is_model_weaknesses=True,
        speech_to_text=False,
        plot=True
        ):
    udi_collector = {}
    chester_collector = {}
    features = []

    if labels is None:
        is_train_model = False
        labels = pd.Series([1] * len(audio_data))

    for audio, sr in zip(audio_data, sample_rates):
        try:
            mfccs = librosa.feature.mfcc(y=audio, sr=sr, n_mfcc=n_mfcc)
            mfccs_mean = np.mean(mfccs.T, axis=0)  # calculate mean of MFCCs
            features.append(mfccs_mean)
        except Exception as e:
            print(f"Error encountered while parsing audio: {e}")

    # create pandas DataFrame
    columns = [f"feat_{i}" for i in range(n_mfcc)]
    df = pd.DataFrame(features, columns=columns)
    df["target"] = labels

    # TODO apply feature scaling if necessary
    if is_train_model:
        chester_collector.update(chester_run(data_spec=Data(df=df, target_column='target'),
                                             is_feature_stats=True,
                                             is_pre_model=is_pre_model,
                                             model_run=model_run, is_model_training=is_train_model,
                                             is_model_weaknesses=is_model_weaknesses,
                                             is_post_model=is_post_model,
                                             plot=plot))

    udi_collector["df"] = df
    udi_collector.update(chester_collector)

    if speech_to_text:
        transcriptions = SpeechToText(audio_data=audio_data, sample_rates=sample_rates).run()
        udi_collector["speech to text"] = transcriptions

    return udi_collector



import cv2


class VideoToGrayScale:
    def __init__(self, video_data):
        self.video_data = video_data
        self.frames = self.video_data.images
        self.grayscale_frames = None
        self.fps = self.video_data.frame_per_second

    def frames_to_grayscale(self):
        """
        Converts a list of frames to grayscale using OpenCV.
        """
        grayscale_frame_list = []

        # loop through each frame in the list
        for frame in self.frames:
            # convert the frame to grayscale
            gray_frame = cv2.cvtColor(frame, cv2.COLOR_BGR2GRAY)
            grayscale_frame_list.append(gray_frame)
        self.grayscale_frames = grayscale_frame_list
        return grayscale_frame_list

    def run(self, play=False):
        if not self.grayscale_frames:
            self.frames_to_grayscale()

        height, width = self.grayscale_frames[0].shape
        size = (width, height)

        video = cv2.VideoWriter('grayscale_output_video.avi', cv2.VideoWriter_fourcc(*'XVID'), self.fps, size,
                                isColor=False)

        for img in self.grayscale_frames:
            video.write(img)

        video.release()

        if play:
            try:
                self.play_grayscale_video()
            except:
                print("Cannot play video on collab, you can read the video and use the vis module to plot images")
        return video

    def play_grayscale_video(self):
        cap = cv2.VideoCapture('grayscale_output_video.avi')

        while cap.isOpened():
            ret, frame = cap.read()

            if not ret:
                break

            cv2.imshow('Grayscale Video', frame)

            if cv2.waitKey(int(1000 / self.fps)) & 0xFF == ord('q'):
                break

        cap.release()
        cv2.destroyAllWindows()



import openai

from databots.utils import MODEL_ENGINE, api_key

openai.api_key = api_key


# todo: handle this
def break_to_subtasks(text):
    temperature = 0.99
    max_tokens = 500
    pre = "The overall task is to write python code "
    text = pre + text
    text = text + "\nDo a code planning. Break it down to subtasks, it will go directly to developer to implement." \
                  "\nif you say to use some model, specify where you get the model first." \
                  "Be clear that the answer should contain python code only."

    response = openai.Completion.create(
        engine=MODEL_ENGINE,
        prompt=text,
        temperature=temperature,
        max_tokens=max_tokens
    )

    code_subtasks = response.choices[0].text + "\n"
    return code_subtasks


def brainstorm_idea(text):
    temperature = 0.99
    max_tokens = 500
    pre = "The overall task is to write python code \n"
    text = pre + text
    text = text + """
    Simulate brainstorm to solve implement this idea with python\n 
    Discuss the purpose and goal of the ideam\n 
    Discuss what would get implemented \n 
    Address concerns\n 
    Discuss potential issues and propose solutions\n 
    Topic 5: Wrap up \n 
    Summarize key points and decisions\n 
    """

    response = openai.Completion.create(
        engine=MODEL_ENGINE,
        prompt=text,
        temperature=temperature,
        max_tokens=max_tokens
    )

    code_subtasks = response.choices[0].text + "\n"
    return code_subtasks



import os

import torch
import torchaudio

from udi.run import run  # Replace 'your_module' with the module name where the 'run' function is defined

# Print torch and torchaudio versions
print(torch.__version__)
print(torchaudio.__version__)

# Set random seed
torch.random.manual_seed(0)

# Set device to use CUDA if available, otherwise use CPU
device = torch.device("cuda" if torch.cuda.is_available() else "cpu")

emotion_path = "/Users/amitosi/PycharmProjects/chester/udi/emotion"

audio_data = []
sample_rates = []

for file in os.listdir(emotion_path):
    if file.endswith(".wav"):
        filepath = os.path.join(emotion_path, file)
        waveform, sample_rate = torchaudio.load(filepath)
        waveform = waveform.to(device)

        # Resample the waveform if sample_rate is not equal to the model's sample rate (16kHz)
        if sample_rate != 16000:
            waveform = torchaudio.functional.resample(waveform, sample_rate, 16000)

        audio_data.append(waveform.squeeze().cpu().numpy())
        sample_rates.append(16000)

# Call the 'run' function with the audio data and sample rates
udi_collector = run(audio_data=audio_data, sample_rates=sample_rates, labels=None, speech_to_text=True)

# Print the predicted transcriptions from the udi_collector dictionary
for i, transcription in enumerate(udi_collector["speech to text"]):
    print(f"Predicted Transcription for file {i + 1}: {transcription}")




def clean_text(text): 
    
    # Remove punctuation
    cleaned_text = text.translate(str.maketrans('', '', string.punctuation))
    
    # Lowercase all words
    cleaned_text = cleaned_text.lower()
    
    # Remove extra spaces
    cleaned_text = cleaned_text.strip()
    return cleaned_text



import pandas as pd
import os

# Load the data
data_path = "/Users/amitosi/PycharmProjects/chester/chester/run_manual_chester/bbc.csv"
bbc_df = pd.read_csv(data_path)

# Remove the unnecessary column and check data types
bbc_df = bbc_df.drop(columns=["Unnamed: 0"])
print("Data types:")
print(bbc_df.dtypes)

# Print the first few rows of data
print("\nSample rows:")
print(bbc_df.head())

# Perform EDA using the provided code snippet
from chester.run import full_run as fr
from chester.run import user_classes as uc

eda_results = fr.run(uc.Data(df=bbc_df))
print("\nEDA Results:")
print(eda_results)


import heapq
from collections import defaultdict

from nltk.probability import FreqDist
from nltk.tokenize import sent_tokenize
from nltk.tokenize import word_tokenize

from chester.cleaning.cleaning_func import clean_text
import pandas as pd


def get_summary_from_df(df, text_column='text', summary_num_sentences=3):
    # Add a new column to the DataFrame with the sentiment of each text
    text = df[text_column].str.cat(sep=". ")
    return summarize_text_textrank(text=text, num_sentences=summary_num_sentences, max_terms=10000)


def get_summary(df, text_columns=None, text_summary=None):
    summary_dict = {}
    # Add a new column to the DataFrame with the sentiment of each text
    if text_columns is None:
        text_columns = ['text']

    if text_summary.id_cols is None:
        for text_col in text_columns:
            text = df[text_col].apply(lambda x: clean_text(
                x,
                remove_punctuation_flag=False,
                remove_numbers_flag=True,
                remove_whitespace_flag=True,
                remove_empty_line_flag=True,
                lowercase_flag=True,
                remove_stopwords_flag=False,
                stopwords=None,
                remove_accented_characters_flag=True,
                remove_special_characters_flag=False,
                remove_html_tags_flag=True
            )).str.cat(sep=". ")  # raw
            summary_dict[text_col] = summarize_text_textrank(text=text,
                                                             num_sentences=text_summary.summary_num_sentences,
                                                             max_terms=text_summary.max_terms)
    else:
        text_columns = list(text_columns)
        # Group the text columns by the ID columns and concatenate the text
        all_cols = list(set(list(text_columns) + list(text_summary.id_cols)))
        aggregated = df[all_cols].groupby(text_summary.id_cols).agg(
            {col: lambda x: '. '.join(clean_text(val,
                                                 remove_punctuation_flag=False,
                                                 remove_numbers_flag=True,
                                                 remove_whitespace_flag=True,
                                                 remove_empty_line_flag=True,
                                                 lowercase_flag=True,
                                                 remove_stopwords_flag=False,
                                                 stopwords=None,
                                                 remove_accented_characters_flag=True,
                                                 remove_special_characters_flag=False,
                                                 remove_html_tags_flag=True) for val in x.dropna()) for col in
             text_columns}).reset_index()
        # Combine id_cols and aggregated text_cols into a single DataFrame
        grouped_text = df[list(text_summary.id_cols)].drop_duplicates().merge(aggregated, on=text_summary.id_cols,
                                                                              how='left')
        grouped_text[text_columns] = grouped_text[text_columns].applymap(
            lambda x: summarize_text_textrank(x, num_sentences=3, max_terms=100))
        summary_dict = grouped_text.set_index(text_summary.id_cols)[text_columns].to_dict('index')

    return summary_dict


def summarize_text_textrank(text, num_sentences, max_terms=100):
    sentences = sent_tokenize(text)
    words = word_tokenize(text.lower())
    word_freq = FreqDist(words)
    ranking = defaultdict(int)
    total_words = 0
    for i, sentence in enumerate(sentences):
        for word in word_tokenize(sentence.lower()):
            if word in word_freq:
                ranking[i] += word_freq[word]
        total_words += len(word_tokenize(sentence))
        if total_words > max_terms:
            num_sentences = i + 1
            break
    top_sentences = heapq.nlargest(num_sentences, ranking, key=ranking.get)
    summary = ' '.join([sentences[i] for i in sorted(top_sentences)])
    return summary



import json

from jinja2 import Template

# Load the template file
with open('/Users/amitosi/PycharmProjects/chester/example_notebooks/template.ipynb') as f:
    template_str = f.read()

# Define the parameters to change
names = ['cpu_small',
         'spoken-arabic-digit',
         'new3s.wc',
         'QSAR-TID-191',
         'letter',
         'w1a',
         'QSAR-TID-11969',
         'segment',
         'mtp',
         'eye_movements',
         'mnist_784',
         'OVA_Lung',
         'QSAR-TID-226',
         'musk',
         'one-hundred-plants-shape',
         'connect-4',
         'one-hundred-plants-margin',
         'pc1',
         'volcanoes-b3',
         'cardiotocography',
         'QSAR-TID-11755',
         'QSAR-TID-11242',
         'ringnorm',
         'steel-plates-fault',
         'QSAR-TID-11',
         'w2a',
         'coil2000',
         'OVA_Breast',
         'Click_prediction_small',
         'QSAR-TID-12687',
         'svmguide1',
         'cpu_small',
         'la2s.wc',
         'vehicle_sensIT',
         'mfeat-zernike',
         'eeg-eye-state',
         'nursery',
         'real-sim',
         'QSAR-TID-11017',
         'quake',
         'QSAR-TID-11631',
         'kin8nm',
         'QSAR-TID-100080',
         'QSAR-TID-250',
         'pendigits',
         'wind',
         'delta_ailerons',
         'analcatdata_halloffame',
         'abalone',
         'w8a',
         'page-blocks',
         'cardiotocography',
         'mv',
         'isolet',
         'cmc',
         'poker',
         'QSAR-TID-10979',
         'QSAR-TID-65',
         'bank32nh',
         'satellite_image',
         'QSAR-TID-10280',
         'volcanoes-d1',
         'baseball',
         'jm1',
         'Click_prediction_small',
         'kr-vs-k',
         'nomao',
         'volcanoes-e1',
         'car',
         'ldpa',
         'SensIT-Vehicle-Combined',
         'spambase',
         'quake',
         'space_ga',
         'kr-vs-kp',
         'CovPokElec',
         'mfeat-morphological',
         'Stagger3',
         'artificial-characters',
         'QSAR-TID-100044']
import os

# Render the template for each parameter value
for name in names:
    # Create a new template with the parameter value
    template = Template(template_str)
    rendered_template = template.render(name=name)

    # Load the rendered template as a JSON object
    nb = json.loads(rendered_template)

    # Replace the "db_name" with the current name
    for cell in nb['cells']:
        if 'db_name' in ''.join(cell.get('source', '')):
            print("original ", cell['source'][0])
            cell['source'][0] = cell['source'][0].replace('db_name', f"'{name}'")
            print("replaced", cell['source'][0])

    # Save the modified notebook to a new file in the "notebooks" subfolder
    if not os.path.exists('notebooks'):
        os.makedirs('notebooks')
    with open(os.path.join('notebooks', f'{name}.ipynb'), 'w') as f:
        json.dump(nb, f)



import openai

from databots.code_to_file.code_to_file import write_code_to_file, run_python_file
from databots.utils import MODEL_ENGINE


def qa(original_task, error, file_path, temperature=0.99, max_tokens=500):
    conversation = f"I got a task: {original_task}\n"
    conversation = conversation + "This is my code:\n"
    code = open(file_path).read() + "\n"
    conversation = conversation + code + "\n"
    conversation = conversation + "And error: \n" + error + "\n"
    conversation = conversation + "Fix the code given this info"

    response = openai.Completion.create(
        engine=MODEL_ENGINE,
        prompt=conversation,
        temperature=temperature,
        max_tokens=max_tokens
    )
    print(conversation)
    qa_response = response.choices[0].text + "\n"
    write_code_to_file(qa_response, file_path)
    return qa_response, run_python_file(file_path)


def qa_fix_comments(comments, file_path, temperature=0.99, max_tokens=500):
    conversation = "Given this code:\n"
    code = open(file_path).read() + "\n"
    conversation = conversation + code + "\n"
    conversation = conversation + "And Comments: \n" + comments + "\n"
    conversation = conversation + "Fix the code given this info"

    response = openai.Completion.create(
        engine=MODEL_ENGINE,
        prompt=conversation,
        temperature=temperature,
        max_tokens=max_tokens
    )
    print(conversation)
    qa_response = response.choices[0].text + "\n"
    write_code_to_file(qa_response, file_path)
    return qa_response, run_python_file(file_path)

# TODO: pip install



import openai

from databots.brainstorming.task_extends import break_to_subtasks
from databots.code_to_file.code_to_file import write_code_to_file, run_python_file
from databots.utils import MODEL_ENGINE


def developer(text, file_path, temperature=0.99, max_tokens=500):
    pre = "Write python code to:"
    text = pre + text + "\n"
    text = text + "You may or may not use the following instructions:\n" + break_to_subtasks(text) + "\n"
    text = text + "In your next message, write the python code only, the rest as comments #comment" + "\n"
    print("Developer instructions: ", text)
    response = openai.Completion.create(
        engine=MODEL_ENGINE,
        prompt=text,
        temperature=temperature,
        max_tokens=max_tokens
    )

    code_response = response.choices[0].text + "\n"
    print("*********************XXXX******")
    print(code_response)
    print("*********************XXXX******")
    write_code_to_file(code_response, file_path)
    return text, run_python_file(file_path)

